<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<gui name="smb4k_shell" version="3">
<MenuBar>
  <Menu noMerge="1" name="file"><text>&amp;File</text>
    <Action name="profiles_menu"/>
    <Separator/>
    <Action name="quit_action"/>
  </Menu>
  <Menu noMerge="1" name="network"><text>&amp;Network</text>
    <ActionList name="network_menu"/>
  </Menu>
  <Menu noMerge="1" name="shares"><text>Sh&amp;ares</text>
    <ActionList name="shares_menu"/>
  </Menu>
  <Menu noMerge="1" name="bookmarks"><text>&amp;Bookmarks</text>
  </Menu>
  <Menu noMerge="1" name="settings"><text>&amp;Settings</text>
    <Merge name ="StandardToolBarMenuHandler"/>
    <Action name="options_show_statusbar"/>
    <Separator/>
    <Merge/>
    <Action name="dock_widgets_menu"/>
    <Action name="shares_view_menu"/>
    <Separator/>
    <Action name="configure_action"/>
  </Menu>
</MenuBar>
  <!-- fullWidth="false" shortens the tool bar, so that it looks nicer. -->
<ToolBar fullWidth="true" name="mainToolBar"><text>Main Toolbar</text>
<!--   <Action name="mount_action"/> -->
  <Separator/>
  <ActionList name="dynamic_list">
  <!-- dynamically loaded by the main window -->
  </ActionList>
  <Separator/>
  <Action name="bookmark_action"/>
  <Separator/>
  <Action name="profiles_menu"/>
  <Separator/>
  <Action name="dock_widgets_menu"/>
  <Action name="shares_view_menu"/>
  <Action name="configure_action"/>
  <Separator/>
  <Action name="quit_action"/>
</ToolBar>
</gui>
